#include <SPI.h>
#include <misakiUTF16.h>

#define CLK_PIN   13  // or SCK
#define MOSI_PIN  11  // or MOSI
#define CS_PIN    10  // or SS

#define DEVS 4 // 連結デバイス数
#define ROWS 8 // 行数
#define COLS 8 // 列数

uint8_t dataBuff[DEVS+1][ROWS] ;
uint8_t workBuff[DEVS+1][ROWS] ;

// ----- MAX7219 コマンド送信 -----

// 1バイトコマンドをN番目のデバイスに送信
void sendCommand(int N,uint8_t addr,uint8_t data) {
  int FN = N -1 ;
  int RN = DEVS-N ;
  digitalWrite(CS_PIN, LOW);
  for (int i=0;i<FN;i++) {
      SPI.transfer(0x00);
      SPI.transfer(0x00);
  }
  SPI.transfer(addr);
  SPI.transfer(data);
  for (int i=0;i<RN;i++) {
      SPI.transfer(0x00);
      SPI.transfer(0x00);
  }
  digitalWrite(CS_PIN, HIGH);
}

// 1バイトコマンドを全てのデバイスに送信
void sendCommand(uint8_t addr, uint8_t data) {
  digitalWrite(CS_PIN, LOW);
  for (int i=0;i<DEVS;i++) {
    SPI.transfer(addr);
    SPI.transfer(data);
  }
  digitalWrite(CS_PIN, HIGH);
}

// バッファデータをN番目のデバイスに送信
void sendBuff(int N) {
  int FN = N -1 ;
  int RN = DEVS-N ;
  memcpy(workBuff,dataBuff,sizeof(workBuff)) ;
  for (int r=0;r<ROWS;r++) {
    digitalWrite(CS_PIN, LOW);
    for (int i=0;i<FN;i++) {
        SPI.transfer(0x00);
        SPI.transfer(0x00);
    }
    uint8_t addr = r + 1 ;
    SPI.transfer(addr);
    SPI.transfer(workBuff[N][r]);
    for (int i=0;i<RN;i++) {
        SPI.transfer(0x00);
        SPI.transfer(0x00);
    }
    digitalWrite(CS_PIN, HIGH);
  }
}

// バッファデータを全てのデバイスに送信
void sendBuff( ) {
  memcpy(workBuff,dataBuff,sizeof(workBuff)) ;
  for (int r=0;r<ROWS;r++) {
    digitalWrite(CS_PIN, LOW);
    for (int d=1;d<=DEVS;d++) {
      uint8_t addr = r + 1 ;
      SPI.transfer(addr);
      SPI.transfer(workBuff[d][r]);
    }
    digitalWrite(CS_PIN, HIGH);
  }
}

// 全部消灯
void clearMAX7219( ) {
  for (int d=0;d<DEVS;d++) {
    for (int r=1;r<=ROWS;r++) {
      uint8_t addr = r ;
      sendCommand(addr,0x00) ;
    }
  }
}

// デバイス初期化
void initMAX7219( ) {
  sendCommand(0x0F,0x00) ;  // 00h:NOMAL MODE  01h:DISPLAY TEST MODE
  sendCommand(0x09,0x00) ;  // 0~7をNo decode Modeに設定
  sendCommand(0x0B,0x07) ;  // スキャン範囲を0~7をNo decode Modeに設定
  sendCommand(0x0A,0x04) ;  // set intensity to a reasonable value : 01h~0Fh ※スキャンデューティー比(明るさ)
  clearMAX7219();
  sendCommand(0x0C,0x01) ;  // 01h:NOMAL OPERATION MODE  00h:SHUTDOWN MODE
}

// ----- 美咲フォント１文字をN番目のバッファに読み出し -----
void sendFont(int N , char * pUTF8) {
  pUTF8 = getFontData(&dataBuff[N][0], pUTF8);  // フォントデータの取得
  sendBuff(N) ;
}

void dispBunner(char * pUTF8) {
  // バナー用パターン作成
  while(*pUTF8) {
    // １文字づつシフト表示
    for (int d=1;d<DEVS;d++) {
      memcpy(&dataBuff[d][0],&dataBuff[d+1][0],ROWS) ;
    }
    pUTF8 = getFontData(&dataBuff[DEVS][0], pUTF8);  // フォントデータの取得
    sendBuff( ) ;
    delay(500) ;
  }  
}

void siftBunner(char * pUTF8) {
  // バナー用パターン作成
  while(*pUTF8) {
    // １Dotづつシフト表示
    Serial.print("pUTF8=") ;
    Serial.println(pUTF8) ;
    pUTF8 = getFontData(&dataBuff[0][0], pUTF8);  // フォントデータの取得

    for (int c=0;c<COLS;c++) {
      for (int d=1;d<=DEVS;d++) {
        int dt = (d == DEVS) ? 0 : d + 1 ;
        for (int r=0;r<ROWS;r++) {
          dataBuff[d][r] <<= 1 ;
          if ((dataBuff[dt][r] & 0x80) != 0) {
            dataBuff[d][r] |= 0x01 ;
          }
        }
      }
      sendBuff( ) ;
      delay(100) ;
      for (int r=0;r<ROWS;r++) {
          dataBuff[0][r] <<= 1 ;
      }
    }
  }  
}

// ----------------------------
void setup() {
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE3);
//  SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));

  pinMode(CS_PIN, OUTPUT);     // Sleva Select
  digitalWrite(CS_PIN, HIGH);  // 解放

  initMAX7219( ) ; 

  sendFont(1,"ゆ") ;
  sendFont(2,"る") ;
  sendFont(3,"プ") ;
  sendFont(4,"ロ") ;
}

void loop() {
  delay(1000) ;
  siftBunner("  みさきフォント表示ＤＥＭＯ ゆるプロ");    
}
